﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.Devices.Gpio;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

// 空白ページのアイテム テンプレートについては、http://go.microsoft.com/fwlink/?LinkId=402352&clcid=0x409 を参照してください

namespace Iotapp01
{
    /// <summary>
    /// それ自体で使用できる空白ページまたはフレーム内に移動できる空白ページ。
    /// </summary>
    public sealed partial class MainPage : Page
    {
        // http://www.atmarkit.co.jp/ait/articles/1511/18/news043.html
        // Windows 10 IoT Coreで始めるIoT入門：第3回 サンプルプログラム

        // Windows IoT Core for Raspberry Pi 2 B : sample program
        // LED R, G, B = GPIO pin #4, #17, #27
        // SW  R, G = GPIO pin #18, #23

        private const int PIN_LED_R =  4;
        private const int PIN_LED_G = 17;
        private const int PIN_LED_B = 27;
        private GpioPin pinLedR, pinLedG, pinLedB;
        private const int PIN_BUTTON_R = 18;
        private const int PIN_BUTTON_G = 23;
        private GpioPin pinButtonR, pinButtonG;
        private bool statusRed, statusGreen, statusBlue;
        private DispatcherTimer timer;

        public MainPage()
        {
            this.InitializeComponent();

            timer = new DispatcherTimer();  // このあたりのコードは Windows IoT のサンプルプログラムより引用
            timer.Interval = TimeSpan.FromMilliseconds(100); // blink 10 times per sec
            timer.Tick += Timer_Tick;
            InitGPIO();
            if (pinLedR != null)
            {
                timer.Start();
            }
        }

        private void InitGPIO()
        {
            var gpio = GpioController.GetDefault();

            if (gpio == null)
            {
                pinLedR = pinLedG = pinLedB = null;
                return;
            }

            // LED出力ピンの初期化
            statusRed = false;
            pinLedR = gpio.OpenPin(PIN_LED_R);
            pinLedR.Write(GpioPinValue.Low);
            pinLedR.SetDriveMode(GpioPinDriveMode.Output);

            statusGreen = false;
            pinLedG = gpio.OpenPin(PIN_LED_G);
            pinLedG.Write(GpioPinValue.Low);
            pinLedG.SetDriveMode(GpioPinDriveMode.Output);

            statusBlue = false;
            pinLedB = gpio.OpenPin(PIN_LED_B);
            pinLedB.Write(GpioPinValue.Low);
            pinLedB.SetDriveMode(GpioPinDriveMode.Output);

            // スイッチ入力ピンの初期化
            pinButtonR = gpio.OpenPin(PIN_BUTTON_R);
            if (pinButtonR.IsDriveModeSupported(GpioPinDriveMode.InputPullUp))
                pinButtonR.SetDriveMode(GpioPinDriveMode.InputPullUp);
            else
                pinButtonR.SetDriveMode(GpioPinDriveMode.Input);
            pinButtonR.DebounceTimeout = TimeSpan.FromMilliseconds(20); // 20msec for debounce
            pinButtonR.ValueChanged += pinButtonR_ValueChanged;

            pinButtonG = gpio.OpenPin(PIN_BUTTON_G);
            if (pinButtonG.IsDriveModeSupported(GpioPinDriveMode.InputPullUp))
                pinButtonG.SetDriveMode(GpioPinDriveMode.InputPullUp);
            else
                pinButtonG.SetDriveMode(GpioPinDriveMode.Input);
            pinButtonG.DebounceTimeout = TimeSpan.FromMilliseconds(20); // 20msec for debounce
            pinButtonG.ValueChanged += pinButtonG_ValueChanged;
        }

        private void Timer_Tick(object sender, object e)
        {
            // statusが同じ場合は再描画しない。という処理は面倒なので、とりあえずスキップ
            // Opaceity が 100%なら点灯、20%なら消灯のつもり
            if (statusRed)
            {
                pinLedR.Write(GpioPinValue.High);
                LedRed.Opacity = 1.0;
            }
            else
            {
                pinLedR.Write(GpioPinValue.Low);
                LedRed.Opacity = 0.2;
            }

            if (statusGreen)
            {
                pinLedG.Write(GpioPinValue.High);
                LedGreen.Opacity = 1.0;
            }
            else
            {
                pinLedG.Write(GpioPinValue.Low);
                LedGreen.Opacity = 0.2;
            }

            if (statusBlue)
            {
                pinLedB.Write(GpioPinValue.High);
                LedBlue.Opacity = 1.0;
            }
            else
            {
                pinLedB.Write(GpioPinValue.Low);
                LedBlue.Opacity = 0.2;
            }
            statusBlue = !statusBlue;
        }

        // ボタンの押下はイベントですぐにキャッチするけど、LEDの点灯や画面の描画は
        // タイマーティックの中で処理するので、少しタイムラグがあります。ひどいコードですが、気にしないでください
        private void pinButtonG_ValueChanged(GpioPin sender, GpioPinValueChangedEventArgs args)
        {
            if (args.Edge == GpioPinEdge.FallingEdge)
            {
                statusGreen = !statusGreen;
            }
        }

        private void pinButtonR_ValueChanged(GpioPin sender, GpioPinValueChangedEventArgs args)
        {
            statusRed = !statusRed;
        }
    }
}
